SystemOrganization addCategory: #'Launcher-Core'!

!MCWorkingCopy class methodsFor: '*launcher-core' stamp: 'kph 2/14/2007 10:18'!
launchFrom: launcher

^launcher launchReportFor: self ! !

!MCWorkingCopy class methodsFor: '*launcher-core' stamp: 'kph 2/13/2007 16:56'!
workingCopies
	^ self allManagers asSortedCollection:
		[ :a :b | a package name <= b package name ]! !

Object subclass: #Launcher
	instanceVariableNames: 'early nextParameterIndex image launchClass nextLaunchDocument'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Launcher-Core'!

!Launcher commentStamp: 'kph 2/14/2007 06:32' prior: 0!
Launcher is an adaptation of the launch process which does not need a re-write of the AutoStart infrastructure.

Classes may implement launchFrom: to participate.

launchFrom is called several times during the startup process, notably before the AutoStart process, and as part of the AutoStart process.

The class effectively registers the time at which it wishes to run by asking for the parameters which it wants from the commandline, if it gets some, then it can run with them. 

two options currently available are
	launcher startParameters "for starting before AutoStart"
	launcher launchParameters "for starting during AutoStart"

with 
	launcher launchParametersOldStyle available if preferred.
!

!Launcher class methodsFor: 'as yet unclassified' stamp: 'kph 2/14/2007 07:13'!
classProjectLauncher

^Smalltalk at: #ProjectLauncher ifAbsent: [ self error: 'ProjectLauncher not present' ]! !

!Launcher class methodsFor: 'as yet unclassified' stamp: 'kph 2/14/2007 07:19'!
initialize

	"Establish whether we are running as part of the squeak initialization process"

	Smalltalk garbageCollect.
	self classProjectLauncher allInstances size = 0 ifTrue: [ ^ self ].

	"we be our own launcher"

	self newLauncher launchImage: SmalltalkImage current withClass: self.

	"for the future"
	
	AutoStart addLauncherFirst: self.
	Smalltalk addToStartUpList: self after: SecurityManager.	
	! !

!Launcher class methodsFor: 'as yet unclassified' stamp: 'kph 2/14/2007 06:47'!
launchFrom: launcher

	^self launchWith: launcher launchParameters			! !

!Launcher class methodsFor: 'as yet unclassified' stamp: 'kph 2/14/2007 09:51'!
launchWith: params

params at: 'TEST' ifPresent: [ :v |
	Transcript cr; show: 'HERE'; cr
].

params at: 'SCRIPTS' ifPresent: [:v | 

	 CodeLoader new loadSourceFiles: (Array with: (v findTokens: ' ')).

].

^true! !

!Launcher class methodsFor: 'as yet unclassified' stamp: 'kph 2/14/2007 07:18'!
newLauncher

^self new beLauncher! !

!Launcher class methodsFor: 'as yet unclassified' stamp: 'kph 2/14/2007 07:18'!
newStarter

^self new beStarter! !

!Launcher class methodsFor: 'as yet unclassified' stamp: 'kph 2/14/2007 07:19'!
startUp: resuming

"we are sent this when starting before autostart"

self newStarter
 launchImage: SmalltalkImage current withClass: self 
! !

!Launcher methodsFor: 'as yet unclassified' stamp: 'kph 2/14/2007 07:14'!
beLauncher

early := false! !

!Launcher methodsFor: 'as yet unclassified' stamp: 'kph 2/14/2007 07:14'!
beStarter

early := true! !

!Launcher methodsFor: 'as yet unclassified' stamp: 'kph 2/13/2007 18:20'!
classes

^ Smalltalk! !

!Launcher methodsFor: 'as yet unclassified' stamp: 'kph 2/14/2007 06:09'!
initialize

nextParameterIndex := 3.
! !

!Launcher methodsFor: 'as yet unclassified' stamp: 'kph 2/14/2007 07:14'!
isLaunch

^self isStart not! !

!Launcher methodsFor: 'as yet unclassified' stamp: 'kph 2/14/2007 07:14'!
isStart

^early ifNil: [ false ]! !

!Launcher methodsFor: 'as yet unclassified' stamp: 'kph 2/14/2007 07:53'!
launchImage: imageOrMock withClass: theFirstLauncherClass  

	image := imageOrMock.
	nextLaunchDocument := nil.
	launchClass := theFirstLauncherClass.
	
	[ launchClass launchFrom: self ] 
		whileTrue: [ 
			launchClass := self nextClassToLaunch.
			launchClass ifNil: [ ^self ] ].
	
	  
			
			
			
			! !

!Launcher methodsFor: 'public' stamp: 'kph 2/14/2007 07:16'!
launchParameters

^self isLaunch ifTrue: [ self nextParametersNewStyle ] ifFalse: [ Dictionary new ]! !

!Launcher methodsFor: 'public' stamp: 'kph 2/14/2007 07:16'!
launchParametersOldStyle

^self isLaunch ifTrue: [ self nextParametersOldStyle ] ifFalse: [ Dictionary new ]! !

!Launcher methodsFor: 'reporting' stamp: 'kph 2/14/2007 10:15'!
launchReportFor: target
 
 | params reports dest |

 params := self launchParameters.

 reports := params at: 'REPORT' ifAbsent: [ ^ self error: 'no report requested' ].

 reports := reports findTokens: ' '.

 dest := params at: 'TO' ifAbsent:  '*.txt'.
 
 reports do: [ : reportSelector |
 	FileStream newFileNamed: (dest copyReplaceAll:'*' with: reportSelector) do: [ :file | 
		self report: reportSelector for: target on: file			
	]	
 ].

 ^true! !

!Launcher methodsFor: 'as yet unclassified' stamp: 'kph 2/13/2007 22:18'!
nextClassToLaunch	
			
	^nextLaunchDocument ifNotNilDo: [:l | self classes at: l asSymbol ifAbsent: [ nil ] ]
	
 ! !

!Launcher methodsFor: 'as yet unclassified' stamp: 'kph 2/13/2007 21:44'!
nextParametersNewStyle

	"this utility extracts the unprocessed parameters into a dictionary from the typical form:
	
	param1=true param2=''hello world'' +param3 -param4

	i.e. items with an = are parameters,
	+ gives param3=true
	- give param4=false

	items without +=- are considered to refer to the next Launch Class or document.
	"

	| key value parameters item |
	
	parameters := Dictionary new.
	
	parameters at: #launcher put: launchClass.
 
	[ (item := image getSystemAttribute: nextParameterIndex) isEmptyOrNil] 
		whileFalse:[
			nextParameterIndex := nextParameterIndex + 1.
			item := item readStream.
			value := nil.
			(item peek = $-) ifTrue: [ value := false. item next  ].
			(item peek = $+) ifTrue: [ value := true.  item next  ].
			key := item upTo: $=.
			value isNil & item atEnd not ifTrue: [ value := item upToEnd ].
			value 
				ifNotNil: [ parameters at: key asUppercase put: value ] 
				ifNil: [ nextLaunchDocument := key. 
					    ^parameters ].
	].
	
	nextLaunchDocument := nil.
	^parameters

			
			
			
			! !

!Launcher methodsFor: 'as yet unclassified' stamp: 'kph 2/13/2007 21:44'!
nextParametersOldStyle

	"this utility extracts the unprocessed parameters into a dictionary from the typical old style form:
	
	KEY value Key2 Value Key3 Value :NextLauncherClass KEY value Key2 value etc.
	
	the ability to specify the nex launcher class by beginning a parameter 
	with a non letter character has been added.
	"

	| key value parameters |
	
	parameters := Dictionary new.
	
	parameters at: #launcher put: launchClass.
 
	[ (key := image getSystemAttribute: nextParameterIndex) isEmptyOrNil] 
		whileFalse:[
			nextParameterIndex := nextParameterIndex + 1.
			key first isLetter ifFalse: [ 
				nextLaunchDocument := key copyWithoutFirst. 
				^ parameters ].
			
			value := image getSystemAttribute: nextParameterIndex.
			nextParameterIndex := nextParameterIndex + 1.
			
			parameters at: key asUppercase put: value. 
	].

	^parameters

			
			
			
			! !

!Launcher methodsFor: 'public' stamp: 'kph 2/14/2007 08:04'!
parameters: dict

"called from AutoStart in order to give us our parameters, we handle our parameters ourself so we can ignore this"! !

!Launcher methodsFor: 'reporting' stamp: 'kph 2/14/2007 11:45'!
report: reportSelector for: item on: stream
	
	(item respondsTo: reportSelector) ifFalse: [ self error: 'requested report not available' ].
	
	(item perform: reportSelector) ifNotNilDo: [ :result | 
		^result isString ifTrue: [ result printOn: stream ] 
		 			   ifFalse: [ result do: [ :each | each printOn: stream. stream cr ] ].
	].
	
	
	 ! !

!Launcher methodsFor: 'public' stamp: 'kph 2/14/2007 07:16'!
startParameters

^self isStart ifTrue: [ self nextParametersNewStyle ] ifFalse: [ Dictionary new ]! !

!Launcher methodsFor: 'public' stamp: 'kph 2/14/2007 08:08'!
startUp 

"we are sent this when starting from autostart"

self beLauncher;
 launchImage: SmalltalkImage current withClass: self class
! !

!ProjectLauncher methodsFor: '*launcher-core' stamp: 'kph 2/14/2007 10:52'!
startUpAfterLogin
	| scriptName loader isUrl |
	self setupFlaps.
	Preferences readDocumentAtStartup ifTrue: [
		HTTPClient isRunningInBrowser ifTrue:[
			self setupFromParameters.
			scriptName _ self parameterAt: 'src'.
			CodeLoader defaultBaseURL: (self parameterAt: 'Base').
		] ifFalse:[
			scriptName _ (SmalltalkImage current getSystemAttribute: 2) ifNil:[''].
			scriptName _ scriptName convertFromSystemString.
			scriptName isEmpty ifFalse:[
				"figure out if script name is a URL by itself"
				isUrl _ (scriptName asLowercase beginsWith:'http://') or:[
						(scriptName asLowercase beginsWith:'file://') or:[
						(scriptName asLowercase beginsWith:'ftp://')]].
				isUrl ifFalse:[
					Smalltalk at: scriptName asSymbol
					ifPresent: [:launchClass | ^ Launcher newLauncher launchImage: SmalltalkImage current withClass: launchClass. ].
					
					scriptName _ 'file:', (FileDirectory default fullNameFor: scriptName) ]
				].
		]. ]
	ifFalse: [ scriptName := '' ].

	scriptName isEmptyOrNil
		ifTrue:[^Preferences eToyFriendly ifTrue: [self currentWorld addGlobalFlaps]].
	loader _ CodeLoader new.
	loader loadSourceFiles: (Array with: scriptName).
	(scriptName asLowercase endsWith: '.pr') 
		ifTrue:[self installProjectFrom: loader]
		ifFalse:[loader installSourceFiles].
! !

!SystemNavigation class methodsFor: '*launcher-core' stamp: 'kph 2/14/2007 10:12'!
launchFrom: launcher

^launcher launchReportFor: self default! !

!SmalltalkImage class methodsFor: '*launcher-core' stamp: 'kph 2/14/2007 10:03'!
launchFrom: launcher

 ^self launchWith: launcher launchParameters! !

!SmalltalkImage class methodsFor: '*launcher-core' stamp: 'kph 2/14/2007 11:26'!
launchWith: params

	| saveAs quit resuming |

	saveAs := params at: 'SAVE' ifAbsent: false.
	quit := params at: 'QUIT' ifAbsent: false.

	(saveAs isKindOf: Boolean) 
		ifFalse: [ resuming := self current saveAs: saveAs.
				  resuming ifTrue: [ ^false ].
				  saveAs := false ].
		
 	resuming := self current snapshot: saveAs andQuit: (quit == true).
	 
	resuming ifTrue: [ ^false ].
	"returning false, stops evaluation of further commandline parameters
	 this will be beneficial for image restart"

	^ true
	! !
Launcher initialize!
